@@ -41,14 +41,11 @@
 (** {2 Elements} *)
 
 type animation_element =
-  [ | `AnimateColor | `AnimateMotion | `AnimateTransform | `Animate | `Set
-  ]
-type descriptive_element = [ | `Desc | `Metdata | `Title ]
+  [ | `AnimateColor | `AnimateMotion | `AnimateTransform | `Animate | `Set ]
+type descriptive_element = [ | `Desc | `Metadata | `Title ]
 type basic_shape_element =
   [ | `Circle | `Ellipse | `Line | `Polygon | `Polyline | `Rect ]
 
-type clipping_path_element = [ | `Path | `Text | basic_shape_element ]
-
 type container_element =
   [
     | `A
@@ -86,26 +83,22 @@ type filter_primitive_element =
 
 type light_source_element =
   [
-    | `FeDiffuseLighting
-    | `FeSpecularLighting
     | `FeDistantLight
     | `FePointLight
     | `FeSpotLight
   ]
 
-type shape_element = [ | `Circle | `Ellipse | `Line | `Polyline | `Polygon | `Rect]
+type shape_element = [ | `Circle | `Ellipse | `Line | `Path | `Polyline | `Polygon | `Rect]
 
 type structural_element = [ | `Defs | `G | `Svg | `Symbol | `Use ]
 
 type text_content_element =
-  [ | `AltGlyph | `TextPath | `Text | `Tref | `Tspan
-  ]
+  [ | `AltGlyph | `TextPath | `Text | `Tref | `Tspan ]
 
 type text_content_child_element =
-  [ | `AltGlyph | `TextPath | `Tref | `Tspan
-  ]
+  [ | `AltGlyph | `TextPath | `Tref | `Tspan ]
 
-type gradient_element = [ | `LinearGradient | `RadialGradient ]
+type gradient_element = [ | `Lineargradient | `Radialgradient ]
 
 type graphics_element =
   [
@@ -125,8 +118,6 @@ type graphics_ref_element = [ | `Image | `Use ]
 
 (** {2 Attributes } *)
 
-type anim_event_attr = [ | `OnBegin | `OnEnd | `OnLoad ]
-
 type conditional_processing_attr =
   [ | `RequiredExtensions | `RequiredFeatures | `SystemLanguage ]
 
@@ -140,12 +131,11 @@ type transfer_attr =
     | `Intercept
     | `Amplitude
     | `Exponent
-    | `Offset__transfer
+    | `Offset_transfer
   ]
 
 type document_event_attr =
-  [ | `OnAbort | `OnError | `OnResize | `OnScroll | `OnUnload | `OnZoom
-  ]
+  [ | `OnAbort | `OnError | `OnResize | `OnScroll | `OnUnload | `OnZoom ]
 
 type filter_primitive_attr = [ | `Height | `Result | `Width | `X | `Y ]
 
@@ -163,14 +153,14 @@ type animation_timing_attr =
     | `Restart
     | `RepeatCount
     | `RepeatDur
-    | `Fill
+    | `Fill_Animation
   ]
 
 type animation_value_attr =
   [ | `CalcMode | `Values | `KeyTimes | `KeySplines | `From | `To | `By
   ]
 
-type animation_addition_attr = [ | `Animation | `Accumulate ]
+type animation_addition_attr = [ | `Additive | `Accumulate ]
 
 type presentation_attr =
   [
@@ -187,7 +177,7 @@ type presentation_attr =
     | `Cursor
     | `Direction
     | `Display
-    | `Document_baseline
+    | `Dominant_baseline
     | `Enable_background
     | `Fill
     | `Fill_opacity
@@ -243,6 +233,7 @@ type graphical_event_attr =
     | `OnFocusOut
     | `OnLoad
     | `OnMouseDown
+    | `OnMouseMove
     | `OnMouseOut
     | `OnMouseOver
     | `OnMouseUp
@@ -287,6 +278,18 @@ type number_optional_number = (number * (number option))
 type percentage = int
 type strings = string list
 
+type color = string
+type icccolor = string
+
+type paint_whitout_icc =
+  [ `None | `CurrentColor
+  | `Color of (color * icccolor option)
+  ]
+
+type paint =
+  [ paint_whitout_icc
+  | `Icc of (iri * paint_whitout_icc option) ]
+
 (* Transformation *)
 type transform =
   | Matrix of (float * float * float * float * float * float)
@@ -474,8 +477,6 @@ type symbol_content =
     | descriptive_element
     | structural_element
     | gradient_element
-    | `LinearGradient
-    | `RadialGradient
     | `A
     | `AltGlyphDef
     | `ClipPath
@@ -737,6 +738,7 @@ type text_content =
     | animation_element
     | descriptive_element
     | text_content_child_element
+    | `PCDATA
     | `A
   ]
 
@@ -750,8 +752,8 @@ type text_attr =
     | `LengthAdjust
     | `X_list
     | `Y_list
-    | `Dx
-    | `Dy
+    | `Dx_list
+    | `Dy_list
     | `Rotate
     | `TextLength
   ]
@@ -763,6 +765,7 @@ type tspan_content =
   [
     | descriptive_element
     | core_attr
+    | `PCDATA
     | `A
     | `AltGlyph
     | `Animate
@@ -781,10 +784,10 @@ type tspan_attr =
     | `Class
     | `Style
     | `ExternalResourcesRequired
-    | `X__list
-    | `Y__list
-    | `Dx
-    | `Dy
+    | `X_list
+    | `Y_list
+    | `Dx_list
+    | `Dy_list
     | `Rotate
     | `TextLength
     | `LengthAdjust
@@ -801,8 +804,8 @@ type tspan_attribute =
     | `ExternalResourcesRequired
     | `X_list
     | `Y_list
-    | `Dx
-    | `Dy
+    | `Dx_list
+    | `Dy_list
     | `Rotate
     | `TextLength
     | `LengthAdjust
@@ -834,6 +837,7 @@ type textpath = [ | `TextPath ]
 type textpath_content =
   [
     | descriptive_element
+    | `PCDATA
     | `A
     | `AltGlyph
     | `Animate
@@ -876,8 +880,8 @@ type altglyph_attr =
     | `ExternalResourcesRequired
     | `X_list
     | `Y_list
-    | `Dx
-    | `Dy
+    | `Dx_list
+    | `Dy_list
     | `GlyphRef
     | `Format
     | `Rotate
@@ -977,7 +981,7 @@ type colorprofile_attr =
     | `Xlink_href
   ]
 
-type lineargradient = [ | `Linear_Gradient ]
+type lineargradient = [ | `Lineargradient ]
 
 (* star *)
 type lineargradient_content =
@@ -1002,7 +1006,7 @@ type lineargradient_attr =
     | `Xlink_href
   ]
 
-type radialgradient = [ | `Radial_Gradient ]
+type radialgradient = [ | `Radialgradient ]
 
 (* star *)
 type radialgradient_content =
@@ -1028,12 +1032,12 @@ type radialgradient_attr =
     | `Xlink_href
   ]
 
-type gradientstop = [ | `Gradient_Stop ]
+type stop = [ | `Stop ]
 
 (* star *)
-type gradientstop_content = [ | `Animate | `Animate_Color | `Set ]
+type stop_content = [ | `Animate | `Animate_Color | `Set ]
 
-type gradientstop_attr =
+type stop_attr =
   [ | core_attr | presentation_attr | `Class | `Style | `Offset
   ]
 
@@ -1238,7 +1242,7 @@ type fecolormatrix_attr =
     | filter_primitive_attr
     | `Class
     | `Style
-    | `Type__fecolor
+    | `Typefecolor
     | `Values
     | `In
   ]
@@ -1454,8 +1458,8 @@ type feoffset_attr =
     | filter_primitive_attr
     | `Class
     | `Style
-    | `Dx_number
-    | `Dy_number
+    | `Dx
+    | `Dy
     | `In
   ]
 
@@ -1660,7 +1664,6 @@ type animatemotion_attr =
     | `ExternalResourcesRequired
     | `Path
     | `KeyPoints
-    | `Path
     | `Rotate
     | `Origin
   ]
@@ -1709,7 +1712,7 @@ type animatetransform_attr =
     | animation_value_attr
     | animation_addition_attr
     | `ExternalResourcesRequired
-    | `Type__animatecolor
+    | `Typeanimatecolor
   ]
 
 type font = [ | `Font ]
@@ -1722,12 +1725,12 @@ type font_attr =
     | `Class
     | `Style
     | `ExternalResourcesRequired
-    | `Horiz___Origin___X
-    | `Horiz___Origin___Y
-    | `Horiz___Adv___X
-    | `Vert___Origin___X
-    | `Vert___Origin___Y
-    | `Vert___Adv___Y
+    | `HorizOriginX
+    | `HorizOriginY
+    | `HorizAdvX
+    | `VertOriginX
+    | `VertOriginY
+    | `VertAdvY
   ]
 
 type font_content =
@@ -1777,14 +1780,14 @@ type glyph_attr =
     | `Class
     | `Style
     | `D
-    | `Horiz___Adv___X
-    | `Vert___Origin___X
-    | `Vert___Origin___Y
-    | `Vert___Adv___Y
+    | `HorizAdvX
+    | `VertOriginX
+    | `VertOriginY
+    | `VertAdvY
     | `Unicode
-    | `Glyph___Name
+    | `GlyphName
     | `Orientation
-    | `Arabic___Form
+    | `ArabicForm
     | `Lang
   ]
 
@@ -1805,7 +1808,7 @@ type missingglyph_content =
     | `Cursor
     | `Filter
     | `Font
-    | `Font___Face
+    | `FontFace
     | `ForeignObject
     | `Image
     | `Marker
@@ -1825,10 +1828,10 @@ type missingglyph_attr =
     | `Class
     | `Style
     | `D
-    | `Horiz___Adv___X
-    | `Vert___Origin___X
-    | `Vert___Origin___Y
-    | `Vert___Adv___Y
+    | `HorizAdvX
+    | `VertOriginX
+    | `VertOriginY
+    | `VertAdvY
   ]
 
 type hkern = [ | `Hkern ]
@@ -1849,21 +1852,21 @@ type fontface_content = [ | descriptive_element | `Font_Face_Src ]
 type fontface_attr =
   [
     | core_attr
-    | `Font___Family
-    | `Font___Style
-    | `Font___Variant
-    | `Font___Weight
-    | `Font___Stretch
-    | `Font___Size
-    | `Unicode___Range
-    | `Units___Per___Em
-    | `Panose___1
+    | `FontFamily
+    | `FontStyle
+    | `FontVariant
+    | `FontWeight
+    | `FontStretch
+    | `FontSize
+    | `UnicodeRange
+    | `UnitsPerEm
+    | `Panose1
     | `Stemv
     | `Stemh
     | `Slope
-    | `Cap___Height
-    | `X___Height
-    | `Accent___Height
+    | `CapHeight
+    | `XHeight
+    | `AccentHeight
     | `Ascent
     | `Descent
     | `Widths
@@ -1872,16 +1875,16 @@ type fontface_attr =
     | `Alphabetic
     | `Mathematical
     | `Hanging
-    | `V___Ideographic
-    | `V___Alphabetic
-    | `V___Mathematical
-    | `V___Hanging
-    | `Underline___Position
-    | `Underline___Thickness
-    | `Strikethrough___Position
-    | `Strikethrough___Thickness
-    | `Overline___Position
-    | `Overline___Thickness
+    | `VIdeographic
+    | `VAlphabetic
+    | `VMathematical
+    | `VHanging
+    | `UnderlinePosition
+    | `UnderlineThickness
+    | `StrikethroughPosition
+    | `StrikethroughThickness
+    | `OverlinePosition
+    | `OverlineThickness
   ]
 
 type fontfacesrc = [ | `Font_Face_Src ]
@@ -1929,5 +1932,3 @@ type foreignobject_attr =
     | `Width
     | `Height
   ]
-
-
